from platform import system

from .ascii_art import image_to_ascii_art
from .Core import core, exceptions, log
from .handwriting import text_to_handwriting
from .mail import send_hmail, send_mail
from .misc import info, playonyt, search, show_history
from .sc import cancel_shutdown, shutdown
from .whats import (
    open_web,
    sendwhatmsg,
    sendwhatmsg_instantly,
    sendwhatmsg_to_group,
    sendwhatmsg_to_group_instantly,
    sendimg_or_video_immediately,
    sendwhatsmsg_to_all,
    sendwhatsdoc_immediately
)

_system = system().lower()
if _system in ("darwin", "windows"):
    from .misc import take_screenshot

if _system == "windows":
    from .remotekit import start_server



class CountryCodeException(Exception):
    """
    Country Code is not present in the Phone Number
    """

    pass


class CallTimeException(Exception):
    """
    Wait time is too short for WhatsApp Web to Open
    """

    pass

class ImageNotFoundException(Exception):
    """
    No image was finded while searching with template matching algorithm
    """
    pass
    

class InternetException(Exception):
    """
    Host machine is not connected to the Internet or the connection Speed is Slow
    """

    pass

class InvalidPhoneNumber(Exception):
    """
    Phone number given is invalid
    """

    pass


class UnsupportedEmailProvider(Exception):
    """
    Email provider used to send the Email is not supported
    """

    pass


class UnableToAccessApi(Exception):
    """unable to access pywhatkit api"""

    pass



import re
import smtplib
from email.message import EmailMessage
from email.mime.text import MIMEText
from typing import Union

from .Core.exceptions import UnsupportedEmailProvider


async def send_mail(
    email_sender: str,
    password: str,
    subject: str,
    message: Union[str, MIMEText],
    email_receiver: str,
) -> None:
    """Send an Email"""

    domain = re.search("(?<=@)[^.]+(?=\\.)", email_sender)

    hostnames = {
        "gmail": "smtp.gmail.com",
        "yahoo": "smtp.mail.yahoo.com",
        "outlook": "smtp.live.com",
        "aol": "smtp.aol.com",
    }

    hostname = None
    for x in hostnames:
        if x == domain.group():
            hostname = hostnames[x]
            break

    if hostname is None:
        raise UnsupportedEmailProvider(f"{domain.group()} is not Supported Currently!")

    with smtplib.SMTP_SSL(hostname, 465) as smtp:
        smtp.login(email_sender, password)

        msg = EmailMessage()
        msg["Subject"] = subject
        msg["From"] = email_sender
        msg["To"] = email_receiver
        msg.set_content(message)

        smtp.send_message(msg)
        print("Email Sent Successfully!")


async def send_hmail(
    email_sender: str, password: str, subject: str, html_code: str, email_receiver: str
) -> None:
    """Send an Email with HTML Code"""

    message = MIMEText(html_code, "html")
    await send_mail(email_sender, password, subject, message, email_receiver)



import os
import platform

try:
    import winerror
except ImportError or ModuleNotFoundError:
    pass

osname = platform.system()


async def shutdown(time: int = 20) -> None:
    """Schedules a Shutdown after the Specified Time"""

    if "window" in osname.lower():
        cont = f"shutdown -s -t {time}"
        error_code = os.system(cont)
        if error_code in [winerror.ERROR_SHUTDOWN_IN_PROGRESS, 1115]:
            print("A Shutdown Process has already been Scheduled!")
        else:
            print(f"Your System will Shutdown in {time} Seconds!")

    elif "linux" in osname.lower():
        cont = f"shutdown -h {time}"
        os.system(cont)
        print(f"Your System will Shutdown in {time} Minutes!")

    elif "darwin" in osname.lower():
        cont = f"shutdown -h -t {time}"
        os.system(cont)
        print(f"Your System will Shutdown in {time} Minutes!")

    else:
        raise Warning(
            f"Available on Windows, Mac and Linux only, can't Execute on {osname}"
        )


def cancel_shutdown() -> None:
    """Cancels the Scheduled Shutdown"""

    if "window" in osname.lower():
        error_code = os.system("shutdown /a")
        if error_code == winerror.ERROR_NO_SHUTDOWN_IN_PROGRESS:
            print(
                "Shutdown Cancellation process has been Aborted! [NO Shutdown Scheduled]"
            )
        else:
            print("Shutdown has been Cancelled!")

    elif "linux" in osname.lower():
        os.system("shutdown -c")
        print("Shutdown has been Cancelled!")

    elif "darwin" in osname.lower():
        os.system("killall shutdown")
        print("Shutdown has been Cancelled!")

    else:
        raise Warning(
            f"Available on Windows, Mac and Linux only, can't Execute on {osname}"
        )



import logging
import sys
import threading

import pyautogui as p
from flask import Flask, request

log = logging.getLogger("werkzeug")
log.setLevel(logging.ERROR)

cli = sys.modules["flask.cli"]
cli.show_server_banner = lambda *x: None

app = Flask("app")

p.FAILSAFE = False
moving = False
drag = False
type_data = ""
old_data = ""
coords = (0, 0)
lastcords = (0, 0)
lstmcord = (0, 0)
lstlen = 0
coords = [(0, 0)]


@app.route("/", methods=["GET", "POST"])
def send() -> str:
    # print("here")
    if request.method == "POST":
        # print(request.get_json())
        return "ok"
    return """<!DOCTYPE html>
<html>
  <head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8"/>
    <meta name='viewport'
     content='width=device-width, initial-scale=1.0, maximum-scale=1.0,
     user-scalable=0' >
    <script type="text/javascript" charset="utf-8">
    function init() {
        var touchzone = document.getElementById("zone");
        touchzone.addEventListener("touchend", clicke, false);
        touchzone.addEventListener("touchmove", handle, false);
        touchzone.addEventListener("touchstart", tostart, false);
        var scrollzone = document.getElementById("scroll");
        scrollzone.addEventListener("touchmove", scrollmove, false);
        scrollzone.addEventListener("touchstart", tostart, false);
    }

function scrollmove(event){
  var a    = event.touches[0].pageX;
  var b     = event.touches[0].pageY;
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  xhttp.open("POST", "scroller", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("a="+a+"&b="+b);
}


function tostart(event){
  var a    = event.touches[0].pageX;
  var b     = event.touches[0].pageY;
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  xhttp.open("POST", "tstart", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("a="+a+"&b="+b);
}

function clicke(event) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  var a = event.changedTouches[0].pageY;
  xhttp.open("POST", "click", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("a="+a);
}

function dragm() {
    var btn = document.getElementById("drag");
    var xhttp = new XMLHttpRequest();
    xhttp.onreadystatechange = function() {
    };
    xhttp.open("POST", "dradhandler", true);
    xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
    xhttp.send();

    if (btn.innerHTML == "Drag mouse"){
        btn.innerHTML = "Move mouse";
    }
    else{
        btn.innerHTML = "Drag mouse";
    }
}

function sendData(a,b,g) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  xhttp.open("POST", "handler", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("a="+a+"&b="+b);
}


 function handle(event) {
  var alpha    = event.touches[0].pageX;
  var beta     = event.touches[0].pageY;
  sendData(alpha,beta);
}


function on_input(){
  var data = document.getElementById("inpfield").value;
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  xhttp.open("POST", "typed", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("data="+data);
}
function enter_but(){
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
  };
  xhttp.open("POST", "enter", true);
  xhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
  xhttp.send("data="+"data");
}




    </script>

    <meta name="" content="">
    <title></title>
  <style>
  .center{
    position: fixed;
    background-color: rgb(230,230,230);
    width:80%;
    height:400px;
    top:300px;
    left:46%;
    color:rgb(200,200,200);
    font-weight:bold;
    transform: translate(-50%,-50%);
    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
   }
   html{
   touch-action:pan-down
   }

   .scrollarea{
    position: fixed;
    background-color: rgb(230,230,230);
    width:30px;
    height:400px;
    top:300px;
    left:92.05%;
    color:rgb(200,200,200);
    font-weight:bold;
    transform: translate(-50%,-50%);
    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
    text-align:center;
   }


  </style>
  </head>
  <body onload="init()">
  <p style="text-align:center;">Type on PC
  <input id="inpfield"type="text" oninput=on_input()></input>
  <button type="button" onclick="enter_but()">Enter</button></p>
  <div style="text-align:center;">
      <button id="drag" type="button" onclick="dragm()">Drag mouse</button>
  </div>
    <div class="center" id="zone">
        <p align="center"><br><br><br><br><br><br>Left click area<br><br><br><br><br><br><hr></p>
        <p align="center"><br>Right click area</p>
    </div>
    <div class="scrollarea" id="scroll">
       <p><br><br><br><br><br>S<br>C<br>R<br>O<br>L<br>L</p>
    </div>
  </body>
</html>"""


@app.route("/handler", methods=["POST"])
def handle() -> str:
    global moving, cordlst, lastcords
    lst = []
    a = request.form["a"]
    b = request.form["b"]
    a = float(a)
    b = float(b)
    moving = True
    coords = (a, b)
    lx, ly = lastcords
    cx, cy = coords
    # print(cx,cy)
    # p.moveRel((cx-lx)/2,(cy-ly)/2)
    threading.Thread(target=lambda: p.moveRel((cx - lx) * 2, (cy - ly) * 2)).start()
    lastcords = coords
    return "1"


@app.route("/scroller", methods=["POST"])
def scrollerr() -> str:
    global moving, cordlst, lastcords2
    a = request.form["a"]
    b = request.form["b"]
    a = float(a)
    b = float(b)
    coords = (a, b)
    lx, ly = lastcords2
    cx, cy = coords
    if cy < ly:
        threading.Thread(target=lambda: p.scroll(50)).start()
    if cy > ly:
        threading.Thread(target=lambda: p.scroll(-50)).start()
    lastcords2 = coords
    return "1"


@app.route("/tstart", methods=["POST"])
def startt() -> str:
    global lastcords, lastcords2
    # print("end")
    a = request.form["a"]
    b = request.form["b"]
    # print(a,b)
    a = float(a)
    b = float(b)
    lastcords = (a, b)
    lastcords2 = (a, b)
    return "1"


@app.route("/click", methods=["POST"])
def do_click() -> str:
    global moving
    if not moving:
        a = request.form["a"]
        # print(a)
        a = float(a)
        if a < 400:
            p.click()
        if a >= 400:
            p.rightClick()

    moving = False
    return "1"


@app.route("/typed", methods=["POST"])
def typeit() -> str:
    global type_data, old_data
    data = request.form["data"]
    type_data = str(data)
    if len(type_data) > len(old_data):
        p.typewrite(type_data[len(type_data) - 1])
    else:
        p.press("backspace", len(old_data) - len(type_data))
    old_data = type_data
    return "1"


@app.route("/enter", methods=["POST"])
def slashN() -> str:
    p.press("enter")
    return "1"


@app.route("/dradhandler", methods=["POST"])
def ghasit_mouse() -> str:
    global drag
    # print(drag)
    if drag == True:
        drag = False
        p.mouseUp()

    else:
        drag = True
        p.mouseDown()

    return "1"


def start_server(port=8000, print_msg=True):
    if print_msg:
        print("Server started at local_ip_of_this_pc:%s" % port)
        print("Print Ctrl+C to exit")
    app.run(host="0.0.0.0", port=port)


# app.run(host='0.0.0.0')
#
#
# def main():
#     app.run(host="192.168.43.17", port=33)
#
#
# if __name__ == "__main__":
#     main()
#     app.run(host='localhost')



from typing import Optional

from PIL import Image


async def image_to_ascii_art(
    img_path: str, output_file: Optional[str] = "pywhatkit_asciiart"
) -> str:
    """Convert an Image to ASCII Art"""

    img = Image.open(img_path).convert("L")

    width, height = img.size
    aspect_ratio = height / width
    new_width = 80
    new_height = aspect_ratio * new_width * 0.55
    img = img.resize((new_width, int(new_height)))

    pixels = img.getdata()

    chars = ["*", "S", "#", "&", "@", "$", "%", "*", "!", ":", "."]
    new_pixels = [chars[pixel // 25] for pixel in pixels]
    new_pixels = "".join(new_pixels)

    new_pixels_count = len(new_pixels)
    ascii_image = [
        new_pixels[index : index + new_width]
        for index in range(0, new_pixels_count, new_width)
    ]
    ascii_image = "\n".join(ascii_image)

    with open(f"{output_file}.txt", "w") as f:
        f.write(ascii_image)
    return ascii_image



import time
import webbrowser as web
from datetime import datetime
from re import fullmatch
from typing import List
from urllib.parse import quote
import pyperclip
import keyboard
import pathlib
import pyautogui as pg
import asyncio
from .Core import core, log, exceptions

pg.FAILSAFE = False


async def main():
    """Check the internet connection."""
    await core.check_connection()


loop = asyncio.get_event_loop()
loop.run_until_complete(main())


async def sendwhatmsg_instantly(
        phone_no: str,
        message: str,
        wait_time: int = 15,
        tab_close: bool = True,
        close_time: int = 3
) -> None:
    """Send a WhatsApp message instantly.
    
    This function opens a new tab in the default web browser, navigates to the WhatsApp web page, and sends a message to the specified phone number.
    
    Parameters:
    message: The message to be sent.
    phone_no: The phone number to send the message to.
    wait_time: The time to wait before sending the message (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the message.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """
    print(phone_no)
    print(await core.check_number(number=phone_no))
    if not await core.check_number(number=phone_no):
        raise exceptions.CountryCodeException("Country Code Missing in Phone Number!")
    phone_no = phone_no.replace(" ", "")
    print(phone_no)
    if not fullmatch(r"^\+?[0-9]{2,4}\s?[0-9]{9,15}", phone_no):
        raise exceptions.InvalidPhoneNumber("Invalid Phone Number.")
    phone_no = phone_no.replace(" ", "")
    web.open(f"https://web.whatsapp.com/send?phone={phone_no}&text={quote(message)}", new=0)
    await asyncio.sleep(wait_time)
    pg.press('enter')
    await asyncio.sleep(close_time)
    if tab_close:
        await core.close_tab(wait_time=close_time)


async def sendwhatmsg(
        phone_no: str = None,
        message: str = None,
        time_hour: int = None,
        time_min: int = None,
        wait_time: int = 15,
        tab_close: bool = False,
        close_time: int = 3,
) -> None:
    """Send a WhatsApp message at a certain time.
    
    This function schedules the sending of a WhatsApp message to a specified phone number at a specified time.
    
    Parameters:
    phone_no: The phone number to send the message to.
    message: The message to be sent.
    time_hour: The hour at which to send the message (in 24-hour format).
    time_min: The minute at which to send the message.
    wait_time: The time to wait before sending the message (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the message.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """
    if not core.check_number(number=phone_no):
        raise exceptions.CountryCodeException("Country Code Missing in Phone Number!")

    phone_no = phone_no.replace(" ", "")
    if not fullmatch(r"^\+?[0-9]{2,4}\s?[0-9]{9,15}", phone_no):
        raise exceptions.InvalidPhoneNumber("Invalid Phone Number.")

    if time_hour not in range(25) or time_min not in range(60):
        raise Warning("Invalid Time Format!")

    current_time = time.localtime()
    left_time = datetime.strptime(
        f"{time_hour}:{time_min}:0", "%H:%M:%S"
    ) - datetime.strptime(
        f"{current_time.tm_hour}:{current_time.tm_min}:{current_time.tm_sec}",
        "%H:%M:%S",
    )

    if left_time.seconds < wait_time:
        raise exceptions.CallTimeException(
            "Call Time must be Greater than Wait Time as WhatsApp Web takes some Time to Load!"
        )

    sleep_time = left_time.seconds - wait_time
    print(
        f"In {sleep_time} Seconds WhatsApp will open and after {wait_time} Seconds Message will be Delivered!"
    )
    await asyncio.sleep(sleep_time)
    await sendwhatmsg_instantly(message, phone_no)
    if tab_close:
        await core.close_tab(wait_time=close_time)


async def sendwhatmsg_to_group(
        group_id: str,
        message: str,
        time_hour: int,
        time_min: int,
        wait_time: int = 15,
        tab_close: bool = False,
        close_time: int = 3,
) -> None:
    """Send a WhatsApp message to a group at a certain time.
    
    This function schedules the sending of a WhatsApp message to a specified group at a specified time.
    
    Parameters:
    group_id: The ID of the group to send the message to.
    message: The message to be sent.
    time_hour: The hour at which to send the message (in 24-hour format).
    time_min: The minute at which to send the message.
    wait_time: The time to wait before sending the message (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the message.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """
    if time_hour not in range(25) or time_min not in range(60):
        raise Warning("Invalid Time Format!")

    current_time = time.localtime()
    left_time = datetime.strptime(
        f"{time_hour}:{time_min}:0", "%H:%M:%S"
    ) - datetime.strptime(
        f"{current_time.tm_hour}:{current_time.tm_min}:{current_time.tm_sec}",
        "%H:%M:%S",
    )

    if left_time.seconds < wait_time:
        raise exceptions.CallTimeException(
            "Call Time must be Greater than Wait Time as WhatsApp Web takes some Time to Load!"
        )

    sleep_time = left_time.seconds - wait_time
    print(
        f"In {sleep_time} Seconds WhatsApp will open and after {wait_time} Seconds Message will be Delivered!"
    )
    await asyncio.sleep(sleep_time)
    await sendwhatmsg_instantly(group_id, message)
    await log.log_message(_time=current_time, receiver=group_id, message=message)
    if tab_close:
        await core.close_tab(wait_time=close_time)


async def sendwhatmsg_to_group_instantly(
        group_id: str,
        message: str,
        wait_time: int = 15,
        tab_close: bool = False,
        close_time: int = 3,
) -> None:
    """Send a WhatsApp message to a group instantly.
    
    This function opens the WhatsApp Web page in a new tab and sends a message to the specified group.
    
    Parameters:
    group_id: The ID of the group to send the message to.
    message: The message to be sent.
    wait_time: The time to wait before sending the message (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the message.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """
    current_time = time.localtime()
    await asyncio.sleep(4)
    await sendwhatmsg_instantly(group_id, message)
    await log.log_message(_time=current_time, receiver=group_id, message=message)

    if tab_close:
        await core.close_tab(wait_time=close_time)


async def sendwhatsmsg_to_all(
        phone_nos: List[str],
        message: str,
        time_hour: int,
        time_min: int,
        wait_time: int = 15,
        tab_close: bool = False,
        close_time: int = 3,
) -> None:
    """Send a WhatsApp message to a list of phone numbers at a certain time.
    
    This function schedules the sending of a WhatsApp message to a list of specified phone numbers at a specified time.
    
    Parameters:
    phone_nos: The list of phone numbers to send the message to.
    message: The message to be sent.
    time_hour: The hour at which to send the message (in 24-hour format).
    time_min: The minute at which to send the message.
    wait_time: The time to wait before sending the message (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the message.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """
    for phone_n in phone_nos:
        await sendwhatmsg(
            phone_n, message, time_hour, time_min, wait_time, tab_close, close_time
        )


async def sendimg_or_video_immediately(
        phone_no: str,
        path: str,
        wait_time: int = 15,
        tab_close: bool = False,
        close_time: int = 3,
) -> None:
    """Send an image or video file via WhatsApp instantly.
    
    This function opens a new tab in the default web browser, navigates to the WhatsApp web page, and sends an image or video file to the specified phone number.
    
    Parameters:
    phone_no: The phone number to send the file to.
    path: The file path of the image or video file to be sent.
    wait_time: The time to wait before sending the file (in seconds).
    tab_close: A flag indicating whether to close the tab after sending the file.
    close_time: The time to wait before closing the tab (in seconds).
    
    Returns:
    None.
    """

    if not await core.check_number(number=phone_no):
        raise exceptions.CountryCodeException("Country Code Missing in Phone Number!")

    phone_no = phone_no.replace(" ", "")
    if not fullmatch(r"^\+?[0-9]{2,4}\s?[0-9]{9,15}", phone_no):
        raise exceptions.InvalidPhoneNumber("Invalid Phone Number.")

    web.open(f"https://web.whatsapp.com/send?phone={phone_no}")
    time.sleep(wait_time)
    await core.find_link()
    time.sleep(1)
    await core.find_photo_or_video()
    if type(path) == str:
        path = pathlib.Path(path)
        pyperclip.copy(str(path.resolve()))
        print("Copied")
    else:
        strn = []
        for paths in path:
            patha = str(pathlib.Path(paths).resolve())
            strn.append(f'"{patha}"')

        print(" ".join(strn))
        pyperclip.copy(" ".join(strn))
    time.sleep(1)
    keyboard.press("ctrl")
    keyboard.press("v")
    keyboard.release("v")
    keyboard.release("ctrl")
    time.sleep(1)
    keyboard.press("enter")
    keyboard.release("enter")
    time.sleep(1)
    keyboard.press("enter")
    keyboard.release("enter")
    if tab_close:
        await core.close_tab(wait_time=close_time)


async def sendwhatsdoc_immediately(
        phone_no: str,
        path: str,
        wait_time: int = 15,
        tab_close: bool = True,
        close_time: int = 3,
) -> None:
    """Send a WhatsApp document instantly.

This function opens a new tab in the default web browser, navigates to the WhatsApp web page, and sends a document to the specified phone number.

Parameters:
phone_no: The phone number to send the document to.
path: The file path of the document to be sent.
wait_time: The time to wait before sending the document (in seconds).
tab_close: A flag indicating whether to close the tab after sending the document.
close_time: The time to wait before closing the tab (in seconds).

Returns:
None.
"""

    if not await core.check_number(number=phone_no):
        raise exceptions.CountryCodeException("Country Code Missing in Phone Number!")

    phone_no = phone_no.replace(" ", "")
    if not fullmatch(r"^\+?[0-9]{2,4}\s?[0-9]{9,15}", phone_no):
        raise exceptions.InvalidPhoneNumber("Invalid Phone Number.")

    web.open(f"https://web.whatsapp.com/send?phone={phone_no}")
    time.sleep(wait_time)
    await core.find_link()
    time.sleep(1)
    await core.find_document()
    if type(path) == str:
        path = pathlib.Path(path)
        pyperclip.copy(str(path.resolve()))
        print("Copied")
    else:
        strn = []
        for paths in path:
            patha = str(pathlib.Path(paths).resolve())
            strn.append(f'"{patha}"')

        print(" ".join(strn))
        pyperclip.copy(" ".join(strn))

    time.sleep(1)
    keyboard.press("ctrl")
    keyboard.press("v")
    keyboard.release("v")
    keyboard.release("ctrl")
    time.sleep(1)
    keyboard.press("enter")
    keyboard.release("enter")
    time.sleep(1)
    keyboard.press("enter")
    keyboard.release("enter")
    if tab_close:
        await core.close_tab(wait_time=close_time)


def open_web() -> bool:
    """Opens WhatsApp Web """

    try:
        web.open("https://web.whatsapp.com")
    except web.Error:
        return False
    else:
        return True



import asyncio
import os
import collections
import time
from concurrent.futures import ThreadPoolExecutor, as_completed
from platform import system
from urllib.parse import quote
from webbrowser import open

import aiohttp
from pyautogui import click, hotkey, moveTo, press, size, typewrite
from pyscreeze import Box, screenshot
import cv2
import numpy as np

from .exceptions import InternetException, ImageNotFoundException

WIDTH, HEIGHT = size()

Box = collections.namedtuple('Box', 'left top width height score')


async def check_number(number: str) -> bool:
    """Checks if the Number is Valid or not"""

    return ("+" in number) or ("_" in number)


async def close_tab(wait_time: int = 2) -> None:
    """Closes the Currently Opened Browser Tab"""

    await asyncio.sleep(wait_time)
    _system = system().lower()
    if _system in ("windows", "linux"):
        hotkey("ctrl", "w")
    elif _system == "darwin":
        hotkey("command", "w")
    else:
        raise Warning(f"{_system} not supported!")
    press("enter")


def find_recent_chat():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    location = locateOnScreen(f"{dir_path}\\data\\searchbar.png")
    try:
        moveTo(location[0] + location[2] / 2, location[1] + location[3])
        click()
    except:
        location = locateOnScreen(f"{dir_path}\\data\\searchbar2.png")
        moveTo(location[0] + location[2] / 2, location[1] + location[3])
        click()


async def findtextbox() -> None:
    """click on text box"""
    dir_path = os.path.dirname(os.path.realpath(__file__))
    location = locateOnScreen(f"{dir_path}\\data\\pywhatkit_smile1.png")
    try:
        moveTo(location[0] + 150, location[1] + 5)
        click()
    except:
        location = locateOnScreen(f"{dir_path}\\data\\pywhatkit_smile.png")
        moveTo(location[0] + 150, location[1] + 5)
        click()


async def find_link():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    print(f"{dir_path}\\data\\link.png")
    linkpaths = ["link.png", "link2.png"]
    locations = [locateOnScreen(f"{dir_path}\\data\\{loc}", grayscale=True, confidence=0.9, multiscale=True) for loc in
                 linkpaths]
    location = None
    y = 0
    for poslink in locations:
        if poslink is not None and poslink[1] > y:
            y = poslink[1]
            location = poslink
    print(location)
    moveTo(location[0] + location[2] / 2, location[1] + location[3] / 2)
    click()


async def find_document():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    location = locateOnScreen(f"{dir_path}\\data\\document.png", confidence=0.8, multiscale=True, grayscale=True)
    print(location)

    moveTo(location[0] + location[2] / 2, location[1] + location[3] / 2)
    click()


async def find_photo_or_video():
    dir_path = os.path.dirname(os.path.realpath(__file__))
    location = locateOnScreen(f"{dir_path}\\data\\photo_or_video.png")
    print(location)
    moveTo(location[0] + location[2] / 2, location[1] + location[3] / 2)
    click()


async def check_connection() -> None:
    """Check the Internet connection of the Host Machine"""

    try:
        async with aiohttp.ClientSession() as session:
            async with session.get("https://google.com") as response:
                status = response.status
                if status < 400:
                    pass

    except:
        raise InternetException(
            "Error while connecting to the Internet. Make sure you are connected to the Internet!"
        )


async def _web(receiver: str, message: str) -> None:
    """Opens WhatsApp Web based on the Receiver"""
    if check_number(number=receiver):
        open(
            "https://web.whatsapp.com/send?phone="
            + receiver
            + "&text="
            + quote(message)
        )
    else:
        open("https://web.whatsapp.com/accept?code=" + receiver)


async def send_message(message: str, receiver: str, wait_time: int) -> None:
    """Parses and Sends the Message"""

    await _web(receiver=receiver, message=message)
    await asyncio.sleep(7)
    click(WIDTH / 2, HEIGHT / 2 + 15)
    await asyncio.sleep(wait_time - 7)
    await _web(receiver=receiver, message=message)
    await asyncio.sleep(7)
    click(WIDTH / 2, HEIGHT / 2)
    await asyncio.sleep(wait_time - 7)
    if not check_number(number=receiver):
        for char in message:
            if char == "\n":
                hotkey("shift", "enter")
            else:
                typewrite(char)
    await findtextbox()
    press("enter")


def locateOnScreen(image, **kwargs):
    """Locate button on screen using cv2.TemplateMatching algorithm

        Parameters
        ----------
        image : str
            The file location of the template image
        grayscale : bool
            Flag to run template matching using grayscale image
        confidence : float
            Confidence Threshold
        mulstiscale : bool
            Flag to run mulstiscale template matching from 1 to 0.8

        Returns
        -------
        Box
            a tuple of (x,y,w,h) of the best match
    """
    screenshotIm = screenshot(region=None)
    boxresult = locateMax_opencv(image, screenshotIm, **kwargs)
    try:
        screenshotIm.fp.close()
    except AttributeError:
        # FROM pyscreeze
        # Screenshots on Windows won't have an fp since they came from
        # ImageGrab, not a file. Screenshots on Linux will have fp set
        # to None since the file has been unlinked
        pass
    return boxresult


def locateMax_opencv(template: str,
                     screenImage: str,
                     grayscale: bool = False,
                     confidence=0.9,
                     multiscale=False) -> Box:
    """Locate button using cv2.TemplateMatching algorithm

        Parameters
        ----------
        template : str
            The file location of the template image
        screenImage : PIL.Image
            The screenshot done using pyscreeze
        grayscale : bool
            Flag to run template matching using grayscale image
        confidence : float
            Confidence Threshold
        mulstiscale : bool
            Flag to run mulstiscale template matching from 1 to 0.8

        Returns
        -------
        Box
            a tuple of (x,y,w,h) of the best match
    """
    confidence = float(confidence)

    template = loadImage(template, grayscale)
    templateH, templateW = template.shape[:2]
    screenImage = loadImage(screenImage, grayscale)

    if (screenImage.shape[0] < template.shape[0] or
            screenImage.shape[1] < template.shape[1]):
        # avoid semi-cryptic OpenCV error below if bad size
        raise ValueError('needle dimension(s) exceed the haystack image or region dimensions')

    if multiscale:
        sizes = [1, 0.9, 0.85, 0.8]
        matchx, matchy = None, None
        with ThreadPoolExecutor() as executor:
            future_to_contour = {executor.submit(cv2.matchTemplate,
                                                 screenImage.copy(),
                                                 cv2.resize(template.copy(), (0, 0), fx=size, fy=size),
                                                 cv2.TM_CCORR_NORMED): size for size in sizes}
            for future in as_completed(future_to_contour):
                _, maxVal, _, maxLoc = cv2.minMaxLoc(future.result())
                if maxVal >= confidence:
                    confidence = maxVal
                    matchx = maxLoc[0]
                    matchy = maxLoc[1]
        if matchx is not None:
            return Box(matchx, matchy, templateW, templateH, maxVal)
        else:
            raise ImageNotFoundException

    result = cv2.matchTemplate(screenImage, template, cv2.TM_CCORR_NORMED)
    (_, maxVal, _, maxLoc) = cv2.minMaxLoc(result)

    if len(result) == 0:
        raise ImageNotFoundException
    if maxVal >= confidence:
        matchx = maxLoc[0]
        matchy = maxLoc[1]

        return Box(matchx, matchy, templateW, templateH, maxVal)
    return


def loadImage(img2load,
              gray: bool):
    if type(img2load) is str:
        assert os.path.exists(img2load)
        return cv2.imread(img2load) if not gray else cv2.cvtColor(cv2.imread(img2load), cv2.COLOR_BGR2GRAY)
    return np.array(img2load) if not gray else cv2.cvtColor(np.array(img2load), cv2.COLOR_BGR2GRAY)



from distutils.core import setup
import pathlib


def readme() -> str:
    with open(r"README.md") as f:
        README = f.read()
    return README


def reqs():
    with open("C:\\Users\\sigir\\PycharmProjects\\AsyncPywhatKit\\requirements.txt", "r") as f:
        requirements = [line.strip() for line in f]
        return requirements


setup(
    name="AsyncPywhatKit",
    packages=['AsyncPywhatKit.src', 'AsyncPywhatKit.src.Core'],
    version="2.1.23",
    setup_requires=['setuptools_scm'],
    license="MIT",
    description="AsyncPywhatKit is a Simple and Powerful WhatsApp Automation Library with many useful Features",
    author="SigireddyBalasai",
    author_email="sigireddybalasai@gmail.com",
    url="https://github.com/SigireddyBalasai/AsyncPywhatKit",
    download_url="https://github.com/SigireddyBalasai/AsyncPywhatKit/archive/refs/tags/1.0.tar.gz",
    keywords=["sendwhatmsg", "info", "playonyt", "search", "watch_tutorial", "async pywhatkit"],
    install_requires=reqs(),
    include_package_data=True,
    long_description=readme(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
)



import os
import time

from .core import check_number


async def format_message(message: str) -> str:
    """Formats the Message to remove redundant Spaces and Newline chars"""
    msg_l = message.split(" ")
    new = []
    for x in msg_l:
        if "\n" in x:
            x = x.replace("\n", "")
            new.append(x) if not len(x) == 0 else None

        elif len(x) != 0:
            new.append(x)

    return " ".join(new)


async def log_message(_time: time.struct_time, receiver: str, message: str) -> None:
    """Logs the Message Information after it is Sent"""

    if not os.path.exists("PyWhatKit_DB.txt"):
        file = open("PyWhatKit_DB.txt", "w+")
        file.close()

    message = await format_message(message)

    with open("PyWhatKit_DB.txt", "a", encoding="utf-8") as file:
        if await check_number(receiver):
            file.write(
                f"Date: {_time.tm_mday}/{_time.tm_mon}/{_time.tm_year}\nTime: {_time.tm_hour}:{_time.tm_min}\n"
                f"Phone Number: {receiver}\nMessage: {message}"
            )
        else:
            file.write(
                f"Date: {_time.tm_mday}/{_time.tm_mon}/{_time.tm_year}\nTime: {_time.tm_hour}:{_time.tm_min}\n"
                f"Group ID: {receiver}\nMessage: {message}"
            )
        file.write("\n--------------------\n")
        file.close()


async def log_image(_time: time.struct_time, path: str, receiver: str, caption: str) -> None:
    """Logs the Image Information after it is Sent"""

    if not os.path.exists("PyWhatKit_DB.txt"):
        file = open("PyWhatKit_DB.txt", "w+")
        file.close()

    caption = format_message(caption)

    with open("PyWhatKit_DB.txt", "a", encoding="utf-8") as file:
        if await check_number(number=receiver):
            file.write(
                f"Date: {_time.tm_mday}/{_time.tm_mon}/{_time.tm_year}\nTime: {_time.tm_hour}:{_time.tm_min}\n"
                f"Phone Number: {receiver}\nImage: {path}\nCaption: {caption}"
            )

        else:
            file.write(
                f"Date: {_time.tm_mday}/{_time.tm_mon}/{_time.tm_year}\nTime: {_time.tm_hour}:{_time.tm_min}\n"
                f"Group ID: {receiver}\nImage: {path}\nCaption: {caption}"
            )
        file.write("\n--------------------\n")
        file.close()



import aiohttp

from AsyncPywhatKit.src.Core import exceptions


async def text_to_handwriting(
    string: str, save_to: str = "pywhatkit.png", rgb: tuple = (0, 0, 0)
) -> None:
    """Convert the given String to Handwritten Characters"""
    async with aiohttp.ClientSession() as session:
        async with session.get(f"https://pywhatkit.herokuapp.com/handwriting?text={string}&rgb={rgb[0]},{rgb[1]},{rgb[2]}") as response:
            status_code = response.status
            if status_code == 200:
                with open(save_to, "wb") as file:
                    file.write(await response.read())
                    file.close()
            elif 400 <= status_code <= 599:
                raise exceptions.UnableToAccessApi("Unable to access Pywhatkit api right now")



import webbrowser as web
from platform import system
import aiohttp
import asyncio
import wikipedia

from .Core import exceptions

if system().lower() in ("windows", "darwin"):
    from PIL import ImageGrab

    async def take_screenshot(
        file_name: str = "pywhatkit_screenshot", delay: int = 2, show: bool = True
    ) -> None:
        """Take Screenshot of the Screen"""

        await asyncio.sleep(delay)
        screen = ImageGrab.grab()
        if show:
            screen.show(title=file_name)
        screen.save(f"{file_name}.png")


async def show_history() -> None:
    """Prints the Log File Generated by the Library"""

    try:
        with open("PyWhatKit_DB.txt", "r") as file:
            content = file.read()
    except FileNotFoundError:
        print("Log File does not Exist!")
    else:
        if len(content) == 0:
            print("No Logs in File!")
        else:
            print(content)


async def info(topic: str, lines: int = 3, return_value: bool = False):
    """Gives Information on the Topic"""

    data = wikipedia.page(topic,auto_suggest=False)
    dataa = data.content
    print(dataa)
    if return_value:
        return dataa


async def playonyt(topic: str, use_api: bool = False, open_video: bool = True) -> str:
    """Play a YouTube Video"""

    if use_api:
        async with aiohttp.ClientSession() as session:
            async with session.get(f"https://pywhatkit.herokuapp.com/playonyt?topic={topic}") as response:
                status_code = response.status
                if status_code == 200:
                    if open_video:
                        web.open(await response.content.decode("ascii"))
                    return await response.read().decode("ascii")
                elif 400 <= status_code <= 599:
                    raise exceptions.UnableToAccessApi(
                "Unable to access pywhatkit api right now"
                    )
    else:
        url = f"https://www.youtube.com/results?q={topic}"
        count = 0
        async with aiohttp.ClientSession() as session:
            async with session.get(url) as cont:
                data = await cont.read()
                data = str(data)
                lst = data.split('"')
                for i in lst:
                    count += 1
                    if i == "WEB_PAGE_TYPE_WATCH":
                        break
                if lst[count - 5] == "/results":
                    raise Exception("No Video Found for this Topic!")

                if open_video:
                    web.open(f"https://www.youtube.com{lst[count - 5]}")
                return f"https://www.youtube.com{lst[count - 5]}"


async def search(topic: str) -> None:
    """Searches About the Topic on Google"""

    link = f"https://www.google.com/search?q={topic}"
    web.open(link)
